Package: dti
Version: 1.5.4
Date: 2023-09-06
Title: Analysis of Diffusion Weighted Imaging (DWI) Data
Authors@R: c(person("Karsten", "Tabelow", role = c("aut", "cre"),
                    email = "karsten.tabelow@wias-berlin.de"),
             person("Joerg", "Polzehl", role = c("aut"),
                    email = "joerg.polzehl@wias-berlin.de"),
             person("Felix", "Anker", role = c("ctb")))
Author: Karsten Tabelow [aut, cre],
  Joerg Polzehl [aut],
  Felix Anker [ctb]
Maintainer: Karsten Tabelow <karsten.tabelow@wias-berlin.de>
Depends: R (>= 3.5.0), awsMethods (>= 1.1-1)
SystemRequirements: gsl
Imports: methods, parallel, adimpro (>= 0.9), aws (>= 2.4.1), rgl,
        oro.nifti (>= 0.3.9), oro.dicom, gsl, quadprog
LazyData: TRUE
Description: Diffusion Weighted Imaging (DWI) is a Magnetic Resonance Imaging
             modality, that measures diffusion of water in tissues like the human
             brain. The package contains R-functions to process diffusion-weighted
             data. The functionality includes diffusion tensor imaging (DTI),
             diffusion kurtosis imaging (DKI), modeling for high angular resolution
             diffusion weighted imaging (HARDI) using Q-ball-reconstruction and
             tensor mixture models, several methods for structural adaptive
             smoothing including POAS and msPOAS, and a streamline fiber tracking
             for tensor and tensor mixture models.
             The package provides functionality to manipulate and visualize results
             in 2D and 3D.
License: GPL (>= 2)
Copyright: This package is Copyright (C) 2005-2020 Weierstrass
        Institute for Applied Analysis and Stochastics.
URL: https://www.wias-berlin.de/research/ats/imaging/
Suggests: covr
RoxygenNote: 6.1.0
NeedsCompilation: yes
Packaged: 2023-09-06 14:21:02 UTC; polzehl
Repository: CRAN
Date/Publication: 2023-09-06 15:30:09 UTC
Built: R 4.2.0; aarch64-apple-darwin20; 2023-09-13 02:51:34 UTC; unix
Archs: dti.so.dSYM
