
<!-- README.md is generated from README.Rmd. Please edit that file -->
`muschellij2` badges: [![AppVeyor Build Status](https://ci.appveyor.com/api/projects/status/github/muschellij2/dti?branch=master&svg=true)](https://ci.appveyor.com/project/muschellij2/dti) [![Travis build status](https://travis-ci.org/muschellij2/dti.svg?branch=master)](https://app.travis-ci.com/muschellij2/dti) [![Coverage status](https://coveralls.io/repos/github/muschellij2/dti/badge.svg)](https://coveralls.io/github/muschellij2/dti)


dti
===

The goal of dti is to provide tools to analysis of Diffusion Weighted Imaging (DWI) Data.

Installation
------------

You can install `dti` from GitHub with:

``` r
# install.packages("devtools")
devtools::install_github("muschellij2/dti")
```
